      SUBROUTINE GUKINE
*.
*...GUKINE   Generate kinematics
*.
*. SEQUENCE  : GCFLAG GCKINE GCONST RCREP  XECGEO XECINT
*. CALLS     : GPRINT GRNDM  GSKINE GSVERT REPORT USKINE
*. CALLED    : GTRIG
*.
*. BANKS L   : None
*. BANKS U   : None
*.
*. AUTHOR    : T.Mashimo
*. VERSION   : 1.13
*. CREATED   : 06-May-00
*. LAST MOD  : 21-Mar-02
*.
*. Modification Log.
*. 21-Mar-02   K.Ozone         (V1.13) improved the energy distribution
*.                             of backward scat'd Compton.
*.                             Removed declare.inc inclusion.
*.                             Modified about -3000<IKINE<-2000.
*. 08-Mar-02   K.Ozone         (V1.12) Adapted BSCON switch
*. 23-Feb-02   K.Ozone         (V1.11) Support for random distribution
*.                             in z direction.(ptype on -2000's)
*.                             Good for cosmic ray simuration.
*. 02-Aug-01   K.Ozone         (V1.10) Support for Comptom scat'd gamma
*.                             distributions
*. 01-Feb-01   T.Mashimo       (V1.10) Support for uniform distributions
*.                             for x and y incident positions.
*. 06-May-00   T.Mashimo       (V1.00) 1st version.
*.
*.**********************************************************************
*.
* For -112 <= ptype <= -1:
* Data card: KINE ptype x y z p1 p2 theta1 theta2 phi1 phi2
* whete theta and phi are in degrees.
*
* For -1112 <= ptype <= -1001:
* Data card: KINE ptype x1 x2 y1 y2 z p1 p2 theta1 theta2 phi
* whete theta and phi are in degrees.
*
* For -2112 <= ptype <= -2001:
* Data card: KINE ptype x1 x2 y z1 z2 p1 p2 theta1 theta2 phi
* whete theta and phi are in degrees.
*
*.**********************************************************************
*.
* HOW TO APPLY BACK COMPTON DISTRIBUTIONS
*
* If p1 > p2, generates along Compton edge distribution presented as
* F(E) = N((E-Ec)^2 + (Ec^2/4),
* where Ec equals p1, the Compton edge energy, and N is a factor.
* The energy window is from p1-p2 to p1, that is to say, p2 is energy range
*
#include "geant321/gconst.inc"
#include "geant321/gcflag.inc"
#include "geant321/gckine.inc"
#include "xecgeo.inc"
#include "xecint.inc"
#include "ntuple.inc"
#include "rcrep.inc"
#include "cmpedg.inc"
*
      LOGICAL INIT
      SAVE    INIT
*
      INTEGER NVERT, NT, I
      REAL    RNDM(3)
      REAL    VERTEX(3), PABS, PLAB(3),
     +        COSTH1, COSTH2, COSTH, SINTH, PHI
*
      DATA INIT   / .FALSE. /
*
      IF (.NOT. INIT) THEN
         INIT = .TRUE.
      END IF
*
      IF (IKINE .EQ. 0) THEN
*
*--      If the 1st parameter of the KINE FFREAD data card is 0,
*--      the user is responsible for generating kinematics
*
         GOTO 500
*
      ELSE IF (IKINE .LT. 0 .AND. IKINE .GE. -1000) THEN
*
*--      A single particle is to be generated.
*--      Its GEANT particle code is -IKINE.
*
         VERTEX(1) = PKINE(1)
         VERTEX(2) = PKINE(2)
         VERTEX(3) = PKINE(3)
         CALL GSVERT(VERTEX, 0, 0, 0, 0, NVERT)
         IF (NVERT .LE. 0) THEN
            WRITE(CHREP, '(''GSVERT: NVERT = '', I10)') NVERT
            CALL REPORT('GUKINE', 1, 'ERROR')
         END IF
*
         IF (BSCON .NE. 0) THEN
*     
*--   Back Compton scattering
*     
            IF (BSCON .EQ. 1) THEN
               PABS = 0.001*HRNDM1(BSCHID)
            ELSE
               CALL FUNLUX(FSPACE, XRAN, XRANLEN)
               PABS = XRAN(1)*0.001
            END IF
            CALL GRNDM(RNDM, 3)
         ELSE
*--      flat energy distribution
            CALL GRNDM(RNDM, 3)
            PABS = 0.001 *( PKINE(4) + RNDM(1)*( PKINE(5) - PKINE(4) ))
         ENDIF
         COSTH1 = COS(PKINE(6)*DEGRAD)
         COSTH2 = COS(PKINE(7)*DEGRAD)
         COSTH = COSTH2 + RNDM(2)*( COSTH1 - COSTH2 )
         IF (ABS(COSTH) .GT. 1.0) COSTH = SIGN(1.0, COSTH)
         SINTH = SQRT(1. - COSTH*COSTH)
         PHI   = DEGRAD*( PKINE(8) + RNDM(3)*( PKINE(9) - PKINE(8) ) )
         PLAB(1) = PABS * SINTH * COS(PHI)
         PLAB(2) = PABS * SINTH * SIN(PHI)
         PLAB(3) = PABS * COSTH
         CALL GSKINE(PLAB, -IKINE, NVERT, 0, 0, NT)
         IF (NT .LE. 0) THEN
            WRITE(CHREP, '(''GSKINE: NT = '', I10)') NT
            CALL REPORT('GUKINE', 2, 'ERROR')
         END IF
*
      ELSE IF (IKINE .LT. -1000 .AND. IKINE .GE. -3000) THEN
*
*--      A single particle is to be generated.
*--      Its GEANT particle code is -IKINE-1000 or -IKINE-2000.
*
         CALL GRNDM(RNDM, 2)
         VERTEX(1) = PKINE(1) + RNDM(1)*( PKINE(2) - PKINE(1) )
         IF (IKINE .LT. -2000) THEN
            VERTEX(2) = PKINE(3)
            VERTEX(3) = PKINE(4) + RNDM(2)*( PKINE(5) - PKINE(4) )
         ELSE
            VERTEX(2) = PKINE(3) + RNDM(2)*( PKINE(4) - PKINE(3) )
            VERTEX(3) = PKINE(5)
          ENDIF
         CALL GSVERT(VERTEX, 0, 0, 0, 0, NVERT)
         IF (NVERT .LE. 0) THEN
            WRITE(CHREP, '(''GSVERT: NVERT = '', I10)') NVERT
            CALL REPORT('GUKINE', 1, 'ERROR')
         END IF
*
         CALL GRNDM(RNDM, 2)
         IF (BSCON .NE. 0) THEN
*     
*--      Back Compton scattering
*
            IF (BSCON .EQ. 1) THEN
               PABS = HRNDM1(BSCHID)*0.001
            ELSE
               CALL FUNLUX(FSPACE, XRAN, XRANLEN)
               PABS = XRAN(1)*0.001
            END IF
         ELSE
            PABS = 0.001 *( PKINE(6) + RNDM(1)*( PKINE(7) - PKINE(6) ) )
         ENDIF
         COSTH1 = COS(PKINE(8)*DEGRAD)
         COSTH2 = COS(PKINE(9)*DEGRAD)
         COSTH = COSTH2 + RNDM(2)*( COSTH1 - COSTH2 )
         IF (ABS(COSTH) .GT. 1.0) COSTH = SIGN(1.0, COSTH)
         SINTH = SQRT(1. - COSTH*COSTH)
         PHI   = DEGRAD*PKINE(10)
         PLAB(1) = PABS * SINTH * COS(PHI)
         PLAB(2) = PABS * SINTH * SIN(PHI)
         PLAB(3) = PABS * COSTH
         CALL GSKINE(PLAB, -IKINE-1000, NVERT, 0, 0, NT)
         IF (NT .LE. 0) THEN
            WRITE(CHREP, '(''GSKINE: NT = '', I10)') NT
            CALL REPORT('GUKINE', 2, 'ERROR')
         END IF
*
      ELSE
         WRITE(CHREP, '(''Invalid IKINE:'', I8)') IKINE
         CALL REPORT('GUKINE', 3, 'ERROR')
      END IF
*
  500 CONTINUE
*
  900 CONTINUE
*
*-- Fill ntuple variables
*
      EVENT = EVENT + 1
      EIN   = PABS
      XIN   = VERTEX(1)
      YIN   = VERTEX(2)
      THIN  = ACOS(COSTH)
*
*-- Initialize variables in /XECINT/
*
      NINTER = 0
      DO 910 I = 1, 3
         X1INTR(I) = 1.E+15
         X1CONV(I) = 1.E+15
  910 CONTINUE
      DEXESM = 0.0
      DEREST = 0.0
      NDEXE  = 0
      DEXESC = 0.0
*
*-- Call user routine
*
      CALL USKINE
*
*-- Print kinematics
*
      IF (IDEBUG .NE. 0) THEN
         CALL GPRINT('VERT', 0)
         CALL GPRINT('KINE', 0)
      END IF
*
  999 CONTINUE
      RETURN
      END
