      SUBROUTINE XSTRAK(XDEPO)
*.
*...XSTRAK   Genereate and transport scintillation light in liq. Xe
*.
*. INPUT     : XDEPO(1-3)  position of energy deposit (x,y,z)
*. INPUT     : XDEPO(4)    deposited energy in GeV
*.
*. SEQUENCE  : GCONST
*.           : RCREP  XECGEO XECPMT XECSCI
*. CALLS     : REPORT XSPOIS XSRNDM XSHITP
*. CALLED    : GUOUT
*.
*. BANKS L   : None
*. BANKS U   : None
*.
*. AUTHOR    : T.Mashimo
*. VERSION   : 1.02
*. CREATED   : 11-Apr-00
*. LAST MOD  : 16-Mar-02
*.
*. Modification Log.
*.
*. 16-Mar-02   K.Ozone         (V1.05) Modified to speed up in loops
*. 21-Feb-02   R.Sawada        (V1.04) Support for photon reflection on
*.                             quartz window
*. 05-Sep-01   W.Ootani        (V1.03) Take into account of liquid xenon
*.                             surface.
*.                             But filled with vacuum instead of Gas Xe
*.                             over liquid Xe surface.
*. 30-Jan-01   T.Mashimo       (V1.02) Fix two bugs;
*.                             (1) When I modified scsim.F, I forgot
*.                             to remove some debris of T. Mori's code;
*.                             simulation of light absorption should
*.                             have been wrong.  (Hopefully nobody has
*.                             ever switched on light absorption.)
*.                             (2) For "XESCRAYL 2", the photon position
*.                             was not advanced when scattering takes
*.                             place.
*. 18-Jan-01   T.Mashimo       (V1.01) Support for "XESCRAYL 2".
*. 11-Apr-00   T.Mashimo       (V1.00) 1st version based on T. Mori's
*.                             SCSIM.
*.
*.**********************************************************************
*.
#include "declare.inc"
#include "geant321/gconst.inc"
#include "xecgeo.inc"
#include "xecpmt.inc"
#include "xecsci.inc"
#include "rcrep.inc"
*
      REAL    XDEPO(4)
*
      INTEGER NPH, IPH,
     +        IWALL, IFACE, IHIT, IADDR, IERR,
     +        IW, ISCAT, I
      REAL    AVNPH
      REAL    X(3), RN(2), CTH, STH, CPH, SPH, V(4), XHIT(3), DIST,
     +        XABSO, XRAYL
      REAL    TH1, TH2
      REAL    CTH1, STH1, CTH2, STH2
      REAL    RFLIDX ! refractive index of quartz
      REAL    KK
      REAL    VNEW(3)
      LOGICAL ROTATE
      INTEGER DIRCT
*
*-- Determine number of scintillation photons to be generated
*
      AVNPH = XDEPO(4)/ESCINT*QEFF      ! == # of photo-electrons
*                                       !  DESTEP > keV by GUSTEP
      PEGEN = PEGEN + AVNPH
      IF (IPOISN .EQ. 1) THEN
         CALL XSPOIS(AVNPH, NPH, 1)
      ELSE
         NPH = NINT(AVNPH)
      END IF
      NPEGEN = NPEGEN + NPH
*
      DO 500 IPH = 1, NPH
*
         RFLG = 1 ! initial photons are in xenon
*
         X(1) = XDEPO(1)
         X(2) = XDEPO(2)
         X(3) = XDEPO(3)
*
 200     CONTINUE
*
         CALL XSRNDM(RN, 2)
         CTH = 2.*RN(1) - 1.
         IF (ABS(CTH) .GE. 1.) THEN
            CTH = SIGN(1., CTH)
            STH = 0.
         ELSE
            STH = SQRT(1.0 - CTH**2)
         END IF
         V(1) = STH*COS(TWOPI*RN(2))  ! photon direction
         V(2) = STH*SIN(TWOPI*RN(2))  !
         V(3) = CTH                   !
         V(4) = VLIGHT                !
*
 300     CONTINUE
*
*-- Transport scintillation light to the wall
*
         CALL XSHITP(X, V, IWALL, IFACE, XHIT, DIST, IHIT, IADDR, IERR)
         IF (IERR .NE. 0) GO TO 500 ! skip if error
*
         ISCAT = 0
         XABSO = 1.E10
         XRAYL = 1.E10
         CALL XSRNDM(RN, 2)
         IF (IABSO .GE. 1) XABSO = -DABSO*ALOG(RN(1))
         IF (LRAYL .GE. 1) XRAYL = -DRAYL*ALOG(RN(2))
*
         IF(RFLG .EQ. 1) THEN   ! if photon inside xenon
*
            IF (XRAYL .GE. XABSO) THEN
               IF (DIST .GT. XABSO) GO TO 500 ! absorbed
            ELSE
               IF (DIST .GT. XRAYL) THEN ! scattered
                  ISCAT = 1
                  IF (LRAYL .EQ. 1) THEN
                     X(1) = X(1) + XRAYL*V(1)
                     X(2) = X(2) + XRAYL*V(2)
                     X(3) = X(3) + XRAYL*V(3)
                     GO TO 200
                  ELSE IF (LRAYL .EQ. 2) THEN
 320                 CONTINUE
                     CALL XSRNDM(RN, 2)
                     CTH = 2.*RN(1) - 1.
                     IF (ABS(CTH) .GE. 1.) CTH = SIGN(1., CTH)
                     STH = SQRT(1.0 - CTH**2)
                     IF (RN(2) .GT. 0.5*(1.+CTH**2)) GO TO 320
*
                     CALL XSRNDM(RN, 1)
                     CPH = COS(TWOPI*RN(1))
                     SPH = SIN(TWOPI*RN(1))
                     VNEW(1) = STH*CPH
                     VNEW(2) = STH*SPH
                     VNEW(3) = CTH
                     CALL GFANG(V, CTH, STH, CPH, SPH, ROTATE)
                     IF (ROTATE) CALL GDROT(VNEW, CTH, STH, CPH, SPH)
                     X(1) = X(1) + XRAYL*V(1)
                     X(2) = X(2) + XRAYL*V(2)
                     X(3) = X(3) + XRAYL*V(3)
                     CALL UCOPY(VNEW, V, 3)
                     GO TO 300
                  END IF
               END IF
            END IF
         END IF
*
*-- If it hits the photo-cathode of a PMT
*
         IF (IHIT .EQ. 1) THEN
            IW = 2*IWALL + IFACE - 2 ! wall number = 1-6
*
*--   forbid any hits from the back of PMT
*
            IF ((IW .EQ. 1 .AND. V(1) .GT. 0)
     +           .OR. (IW .EQ. 2 .AND. V(1) .LT. 0)
     +           .OR. (IW .EQ. 3 .AND. V(2) .GT. 0)
     +           .OR. (IW .EQ. 4 .AND. V(2) .LT. 0)
     +           .OR. (IW .EQ. 5 .AND. V(3) .GT. 0)
     +           .OR. (IW .EQ. 6 .AND. V(3) .LT. 0)) THEN
               GO TO 500
            END IF
*
*--- which face does the photon hit on ?
*--- 1:right, 2:left, 3:top, 4:bottom, 5:front, 6:back
*
            IF (QREFON .EQ. 1) THEN
               DIRCT = INT((IW-1)/2+1)
               CTH1 = ABS(V(DIRCT))
               IF (ACOS(CTH1) .GT. CTHETA) THEN ! total reflection
                  V(DIRCT) = -V(DIRCT)
                  GO TO 300
               ELSE
                  STH1 = SQRT(1-CTH1**2)
                  STH2 = STH1*(RFRIDX/QRFIDX) ! Snell's law
                  CTH2 = SQRT(1-STH2**2)
                  RFLIDX = (SIN(ASIN(CTH1)-ASIN(CTH2))**2
     +                 / SIN(ASIN(CTH1)+ASIN(CTH2))**2
     +                 + TAN(ASIN(CTH1)-ASIN(CTH2))**2
     +                 / TAN(ASIN(CTH1)+ASIN(CTH2))**2)*0.5
                  CALL XSRNDM(RN, 1)
                  IF (RN(1) .LT. RFLIDX) THEN
                     V(DIRCT) = -V(DIRCT) ! normal reflection
                     GO TO 300
                  ELSE
                     GO TO 330
                  END IF
               END IF
            END IF
 330        NPEALL        = NPEALL + 1
            NPEWAL(IW)    = NPEWAL(IW) + 1
            NPEPMT(IADDR) = NPEPMT(IADDR) + 1
            GO TO 500
*
*-- If it hits xenon surface
*
         ELSE IF ((IHIT .EQ. 2) .AND. (FILL .LT. 1.0)) THEN
            X(1) = XHIT(1)
            X(2) = XHIT(2)
            X(3) = XHIT(3)
            IF (RFLG .EQ. 1) THEN ! xenon->vacuum or xenon->xenon
               CTH1 = V(2)
               IF (ACOS(CTH1) .GT. CTHETA) THEN ! total reflection
                  V(2) = -V(2)
                  GO TO 300
               ELSE
                  STH1 = SQRT(1-CTH1**2)
                  STH2 = STH1*RFRIDX ! Snell's law
                  CTH2 = SQRT(1-STH2**2)
                  RFLIDX = (SIN(ASIN(CTH1)-ASIN(CTH2))**2
     +                 / SIN(ASIN(CTH1)+ASIN(CTH2))**2
     +                 + TAN(ASIN(CTH1)-ASIN(CTH2))**2
     +                 / TAN(ASIN(CTH1)+ASIN(CTH2))**2)*0.5
                  CALL XSRNDM(RN, 1)
                  IF (RN(1) .LT. RFLIDX) THEN
                     V(2) = -V(2) ! normal reflection
                     GO TO 300 
                  ELSE
                     KK   = -CTH1+STH1*STH2/CTH2
                     V(1) = V(1)/SQRT(1+KK**2+2*KK*V(2))
                     V(2) = (V(2)+KK)/SQRT(1+KK**2+2*KK*V(2))
                     V(3) = V(3)/SQRT(1+KK**2+2*KK*V(2))
                     RFLG = 0
                     GO TO 300
                  END IF
               END IF
            ELSE                ! vacuum->xenon
               CTH1 = V(2)
               STH1 = SQRT(1-CTH1**2)
               STH2 = STH1/RFRIDX ! Snell's law
               CTH2 = SQRT(1-STH2**2)
               KK   = -CTH1+STH1*STH2/CTH2
               V(1) = V(1)/SQRT(1+KK**2-2*KK*V(2))
               V(2) = (V(2)-KK)/SQRT(1+KK**2-2*KK*V(2))
               V(3) = V(3)/SQRT(1+KK**2-2*KK*V(2))
               RFLG = 1
               GO TO 300 
            END IF

*--   If it hits a place outside PMT  -- reflection?
         ELSE IF (IREFL .EQ. 1) THEN
            CALL XSRNDM(RN, 1)
            IF (RN(1) .LT. EREFL) THEN
               V(IWALL) = -V(IWALL) ! regular reflection
               X(1) = XHIT(1)
               X(2) = XHIT(2)
               X(3) = XHIT(3)
               GO TO 300        ! reflected
            ELSE
               GO TO 500
            END IF
         END IF

 500  CONTINUE
*     
 999  CONTINUE
       RETURN
      END
